@@ -180,10 +180,10 @@ RG.MA <- function(object) {
 normalizeWithinArrays <- function(object,layout=object$printer,method="printtiploess",weights=object$weights,span=0.3,iterations=4,controlspots=NULL,df=5,robust="M",bc.method="subtract",offset=0)
 #	Within array normalization
 #	Gordon Smyth
-#	2 March 2003.  Last revised 29 July 2005.
+#	2 March 2003.  Last revised 26 August 2005.
 {
 	if(!is(object,"MAList")) object <- MA.RG(object,bc.method=bc.method,offset=offset)
-	choices <- c("none","median","loess","printtiploess","composite","robustspline")
+	choices <- c("none","median","loess","printtiploess","composite","control","robustspline")
 	method <- match.arg(method,choices)
 	if(method=="none") return(object)
 	narrays <- ncol(object$M)
@@ -250,6 +250,25 @@ normalizeWithinArrays <- function(object,layout=object$printer,method="printtipl
 				}
 			}
 		},
+		control = {
+			if(is.null(layout)) stop("Layout argument not specified")
+			if(is.null(controlspots)) stop("controlspots argument not specified")
+			ntips <- layout$ngrid.r * layout$ngrid.c
+			nspots <- layout$nspot.r * layout$nspot.c
+			for (j in 1:narrays) {
+				y <- object$M[,j]
+				x <- object$A[,j]
+				f <- is.finite(y) & is.finite(x)
+				if(!is.null(weights)) {
+					w <- weights[,j]
+					f <- f & is.finite(w)
+				}
+				y[!f] <- NA
+				fit <- loess(y~x,weights=w,span=span,subset=controlspots,na.action=na.exclude,degree=1,surface="direct",family="symmetric",trace.hat="approximate",iterations=iterations)
+				y[f] <- y[f]-predict(fit,newdata=x[f])
+				object$M[,j] <- y
+			}
+		},
 		robustspline = {
 			if(is.null(layout)) stop("Layout argument not specified")
 			for (j in 1:narrays)
